library tim_ui_kit_calling_plugin;

import 'package:flutter/material.dart';
import 'package:tencent_im_sdk_plugin/enum/offlinePushInfo.dart';
import 'package:tencent_im_sdk_plugin/models/v2_tim_user_full_info.dart';
import 'package:tencent_im_sdk_plugin/tencent_im_sdk_plugin.dart';
import 'package:tim_ui_kit_calling_plugin/enum/tim_uikit_trtc_call_conv_type.dart';
import 'package:tim_ui_kit_calling_plugin/enum/tim_uikit_trtc_calling_scence.dart';
import 'package:tim_ui_kit_calling_plugin/model/TIMUIKitCallingListener.dart';
import 'package:tim_ui_kit_calling_plugin/model/TRTCCalling.dart';
import 'package:tim_ui_kit_calling_plugin/view/tim_uikit_trtc.dart';

export 'package:tencent_trtc_cloud/trtc_cloud_def.dart';
export 'package:tencent_trtc_cloud/trtc_cloud_video_view.dart';

class TUICalling {
  static bool _enableCustomViewRoute = false;
  static bool _enableFloatingWindow = false;
  static GlobalKey<NavigatorState> navigatorKey = GlobalKey<NavigatorState>();
  static late V2TimUserFullInfo _loginUserInfo;
  static late TRTCCalling _tRTCCallingService;
  static bool _isInit = false;
  static late TUICallingListener _onRtcListener;

  init(
      {required int sdkAppID,
      required String userID,
      required String userSig}) async {
    _tRTCCallingService = await TRTCCalling.sharedInstance();
    _tRTCCallingService.login(sdkAppID, userID, userSig);
    _loginUserInfo = (await _getUserInfo([userID]))!.first;
    _registerListener();
    _isInit = true;
  }

  enableCustomViewRoute({bool isEnable = true}) {
    _enableCustomViewRoute = isEnable;
  }

  enableFloatingWindow({bool isEnable = true}) {
    _enableFloatingWindow = isEnable;
  }

  _registerListener() {
    _onRtcListener = TUICallingListener(
      onInvited: (params) async {
        if (_enableCustomViewRoute) {
          return;
        }
        final inviterUserID = params.sponsor;

        // 被邀请方用户ID数组，不包含自己
        final inviteUserIDs = params.userIDList;

        // 通话场景 语音通话/视频通话
        final callScenes = params.callType;

        /// 是否群通话
        final isFromGroup = params.isFromGroup;

        final groupID = params.groupID;

        /// 邀请方用户信息
        final inviterUserInfo = await _getUserInfo([inviterUserID]);

        if (_loginUserInfo == null) {
          return;
        }

        /// 被邀请方用户信息
        List<V2TimUserFullInfo> inviteUsersInfoList = (inviteUserIDs.isNotEmpty
            ? [_loginUserInfo, ...?await _getUserInfo(inviteUserIDs)]
            : [_loginUserInfo]);

        /// 是否主动邀请发起通话
        const isInitiationCalling = false;

        _buildCallingOverlay(
            isFromGroup ? ConvType.group : ConvType.c2c,
            inviteUsersInfoList,
            inviterUserInfo!.first,
            callScenes == "audioCall"
                ? CallingScenes.Audio
                : CallingScenes.Video,
            isInitiationCalling,
            groupID);
      },
    );
    _tRTCCallingService.registerListener(_onRtcListener);
  }

  destroy() {
    if (!_isInit) {
      throw Exception("not init");
    }
    _tRTCCallingService.destroy();
  }

  Future<List<V2TimUserFullInfo>?> _getUserInfo(List<String> userIDList) async {
    final res = await TencentImSDKPlugin.v2TIMManager
        .getUsersInfo(userIDList: userIDList);
    return res.data;
  }

  Future<TRTCCalling> sharedInstance() {
    return TRTCCalling.sharedInstance();
  }

  Future<void> call(String userId, CallingScenes type,
      [OfflinePushInfo? offlinePushInfo]) async {
    if (_enableCustomViewRoute) {
      return;
    }

    /// 被邀请方用户信息
    final inviteUsersInfoList = await _getUserInfo([userId]);

    /// 邀请方
    final inviterUserInfo = _loginUserInfo;

    if (inviterUserInfo == null) {
      return;
    }

    /// 1:语音通话 2:视频通话
    final callScenes = type;

    /// 是否是主动发起通话
    const isInitiationCalling = true;

    _buildCallingOverlay(ConvType.c2c, inviteUsersInfoList, inviterUserInfo,
        callScenes, isInitiationCalling, null, offlinePushInfo);
  }

  Future<void> groupCall(
      List<String> userIdList, CallingScenes type, String? groupId) async {
    if (_enableCustomViewRoute) {
      return;
    }

    /// 被邀请方用户信息
    final inviteUsersInfoList = await _getUserInfo(userIdList);

    /// 邀请方
    final inviterUserInfo = _loginUserInfo;

    if (inviteUsersInfoList == null || inviterUserInfo == null) {
      return;
    }

    /// 语音通话:CallingScenes.Video
    /// 视频通话:CallingScenes.Audio
    final callScenes = type;

    /// 是否是主动发起通话
    const isInitiationCalling = true;

    if (inviteUsersInfoList.length + 1 > 9) {
      return;
    }

    _buildCallingOverlay(ConvType.group, inviteUsersInfoList, inviterUserInfo,
        callScenes, isInitiationCalling, groupId);
  }

  _buildCallingOverlay(convType, inviteUsersInfoList, inviterUserInfo,
      callScenes, isInitiationCalling,
      [String? groupId, OfflinePushInfo? offlinePushInfo]) {
    OverlayEntry? _entry;

    final entry = OverlayEntry(builder: (context) {
      return TIMUIKitTRTCContainer(
          convType: convType,
          inviteUsersInfoList: inviteUsersInfoList!,
          inviterUserInfo: inviterUserInfo,
          callScenes: callScenes,
          isInitiationCalling: isInitiationCalling,
          groupID: groupId,
          enableFloatingWindow: _enableFloatingWindow,
          offlinePushInfo: offlinePushInfo,
          close: () {
            _entry?.remove();
          });
    });
    _entry = entry;

    navigatorKey.currentState?.overlay?.insert(entry);
  }

  setCallingListener(TUICallingListener listener) {
    _tRTCCallingService.registerListener(listener);
  }

  removeCallingListener(TUICallingListener listener) {
    _tRTCCallingService.unRegisterListener(listener);
  }
}
