// ignore_for_file: prefer_const_literals_to_create_immutables

import 'dart:async';

import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:tencent_im_sdk_plugin/enum/offlinePushInfo.dart';
import 'package:tencent_im_sdk_plugin/models/v2_tim_user_full_info.dart';
import 'package:tencent_trtc_cloud/trtc_cloud_def.dart';
import 'package:tencent_trtc_cloud/trtc_cloud_video_view.dart';
import 'package:tim_ui_kit_calling_plugin/enum/tim_uikit_trtc_call_conv_type.dart';
import 'package:tim_ui_kit_calling_plugin/enum/tim_uikit_trtc_call_status_enum.dart';
import 'package:tim_ui_kit_calling_plugin/enum/tim_uikit_trtc_call_type_enum.dart';
import 'package:tim_ui_kit_calling_plugin/enum/tim_uikit_trtc_calling_scence.dart';
import 'package:tim_ui_kit_calling_plugin/model/TIMUIKitCallingListener.dart';
import 'package:tim_ui_kit_calling_plugin/model/TRTCCalling.dart';
import 'package:tim_ui_kit_calling_plugin/view/c2c/tim_uikit_trtc_c2c_extent_button.dart';
import 'package:tim_ui_kit_calling_plugin/view/c2c/utils.dart';

class TIMUIKitTRTCc2cCalling extends StatefulWidget {
  final bool isInitiationCalling;
  final ConvType convType;
  final V2TimUserFullInfo inviterUserInfo;
  final List<V2TimUserFullInfo> inviteUsersInfoList;
  final CallingScenes callScenes;
  final Function close;
  final OfflinePushInfo? offlinePushInfo;
  const TIMUIKitTRTCc2cCalling({
    // 判断群组通话还是c2c通话
    required this.convType,
    // 邀请人信息
    required this.inviterUserInfo,
    // 被邀请人信息,c2c情况下 只取第一个
    required this.inviteUsersInfoList,

    /// 语音通话 :CallingScenes.Audio
    /// 视频通话:CallingScenes.Video
    required this.callScenes,
    // 是否主动发起通话
    required this.isInitiationCalling,
    this.offlinePushInfo,
    Key? key,
    required this.close,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() => TIMUIKitTRTCc2cCallingState();
}

class TIMUIKitTRTCc2cCallingState extends State<TIMUIKitTRTCc2cCalling> {
  CallingScenes _callingScenes = CallingScenes.Audio;
  CallStatus _currentCallStatus = CallStatus.calling;
  CallTypes _currentCallType = CallTypes.Type_Call_Someone;

  //已经通话时长
  String _hadCallingTime = "00:00";
  late DateTime _startAnswerTime;
  bool _isCameraOff = false;
  bool _isHandsFree = true;
  bool _isMicrophoneOff = false;
  bool _isFrontCamera = true;
  int _bigVideoViewId = -1;
  Timer? _hadCalledCalcTimer;

  late int _smallVideoViewId;
  // 头像上方便宜高度
  double _smallViewTop = 128;
  // 头像偏移右边高度
  double _smallViewRight = 20;
  //为false的时候，在已接听状态的时候。小画面显示本地视频，大画面显示远端视频。
  bool isChangeBigSmallVideo = false;

  //远端画面可见不可见
  bool _remoteUserAvailable = true;
  late V2TimUserFullInfo _remoteUserInfo;

  late TRTCCalling _tRTCCallingService;
  late TUICallingListener _callingListener;

  @override
  void initState() {
    super.initState();
    initRemoteInfo();
    initTrtc();
  }

  initTrtc() async {
    _tRTCCallingService = await TRTCCalling.sharedInstance();
    _callingListener = TUICallingListener(
        onCallEnd: () {
          showMessageTips("结束通话", stopCameraAndFinish, context);
        },
        onCallingTimeout: (params) {
          showMessageTips("通话超时未应答", stopCameraAndFinish, context);
        },
        onReject: (params) {
          showMessageTips("拒绝通话", stopCameraAndFinish, context);
        },
        onUserLeave: (params) {
          showMessageTips("对方已挂断", stopCameraAndFinish, context);
        },
        onUserEnter: (params) {
          handleOnUserAnswer();
        },
        onError: (params) {
          showMessageTips(
              "发生错误:" + params.errCode.toString() + "," + params.errMsg,
              stopCameraAndFinish,
              context);
        },
        onNoResp: (params) {
          showMessageTips("无响应", stopCameraAndFinish, context);
        },
        onLineBusy: (params) {
          showMessageTips("忙线", stopCameraAndFinish, context);
        },
        onKickedOffline: () {
          showMessageTips("你被踢下线了", stopCameraAndFinish, context);
        },
        onCallingCancel: () {
          showMessageTips("取消了通话", stopCameraAndFinish, context);
        },
        onEnterRoom: (time) {},
        onUserVideoAvailable: (params) {
          handleOnUserVideoAvailable(params);
        });
    _tRTCCallingService.registerListener(_callingListener);
  }

  initRemoteInfo() async {
    _safeSetState(() {
      _remoteUserInfo = widget.isInitiationCalling
          ? widget.inviteUsersInfoList.first
          : widget.inviterUserInfo;
      _currentCallType = widget.isInitiationCalling
          ? CallTypes.Type_Call_Someone
          : CallTypes.Type_Being_Called;
      _callingScenes = widget.callScenes;
      Future.delayed(const Duration(microseconds: 100), () {
        if (_currentCallType == CallTypes.Type_Call_Someone) {
          _tRTCCallingService.call(
              _remoteUserInfo.userID ?? "",
              _callingScenes == CallingScenes.Video
                  ? TRTCCalling.typeVideoCall
                  : TRTCCalling.typeAudioCall,
              widget.offlinePushInfo);
        }
      });
    });
  }

  @override
  dispose() {
    if (_hadCalledCalcTimer != null) {
      _hadCalledCalcTimer!.cancel();
    }
    _tRTCCallingService.unRegisterListener(_callingListener);
    super.dispose();
  }

  // 当用户接到视频电话时
  handleOnUserVideoAvailable(params) async {
    if (params["userId"].toString() == _remoteUserInfo.userID) {
      _safeSetState(() {
        _remoteUserAvailable = params["available"] as bool;
      });
    }
  }

  stopCameraAndFinish(context) {
    // 保证延时执行时context存在
    _tRTCCallingService.setMicMute(true);
    _tRTCCallingService.closeCamera();
    widget.close();
  }

  showMessageTips(String msg, Function callback, context) {
    Fluttertoast.showToast(
      msg: msg,
      gravity: ToastGravity.BOTTOM,
      timeInSecForIosWeb: 1,
      textColor: Colors.white,
      backgroundColor: Colors.black,
    );

    context == null ? callback() : callback(context);
  }

  //麦克风启用禁用
  onMicrophoneTap() {
    _tRTCCallingService.setMicMute(!_isMicrophoneOff);
    setState(() {
      _isMicrophoneOff = !_isMicrophoneOff;
    });
  }

  //前后摄像头切换
  _onSwitchCamera() {
    _tRTCCallingService.switchCamera(!_isFrontCamera);
    _safeSetState(() {
      _isFrontCamera = !_isFrontCamera;
    });
  }

  //摄像头启用禁用
  onCameraTap() async {
    if (!_isCameraOff) {
      await _tRTCCallingService.closeCamera();
    } else {
      //为false的时候，在已接听状态的时候。小画面显示本地视频，大画面显示远端视频。未接听的状态下依然使用大端显示
      if (isChangeBigSmallVideo || _currentCallStatus == CallStatus.calling) {
        await _tRTCCallingService.openCamera(_isFrontCamera, _bigVideoViewId);
      } else {
        await _tRTCCallingService.openCamera(_isFrontCamera, _smallVideoViewId);
      }
    }
    _safeSetState(() {
      _isCameraOff = !_isCameraOff;
    });
  }

  onHandsfreeTap() {
    _tRTCCallingService.setHandsFree(!_isHandsFree);
    setState(() {
      _isHandsFree = !_isHandsFree;
    });
  }

  _callIngTimeUpdate() {
    _hadCalledCalcTimer =
        Timer.periodic(const Duration(seconds: 1), (Timer timer) {
      DateTime now = DateTime.now();
      Duration duration = now.difference(_startAnswerTime);
      _safeSetState(() {
        _hadCallingTime = TRTCC2cUtils.getDurationTimeString(duration);
      });
    });
  }

  onSwitchAudioTap() {
    _tRTCCallingService.closeCamera();
    _safeSetState(() {
      _callingScenes = CallingScenes.Audio;
    });
  }

  //用户接听
  handleOnUserAnswer() {
    _startAnswerTime = DateTime.now();
    _safeSetState(() async {
      _currentCallStatus = CallStatus.answer;
      _hadCallingTime = "00:00";
      if (_bigVideoViewId != -1) {
        await _tRTCCallingService.startRemoteView(
          _remoteUserInfo.userID!,
          TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_BIG,
          _bigVideoViewId,
        );
        // await _tRTCCallingService.updateRemoteView(_remoteUserInfo.userID!,
        //     TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_BIG, _bigVideoViewId);
      }
    });
    _callIngTimeUpdate();
  }

  //挂断
  onHangUpCall() async {
    _tRTCCallingService.closeCamera();
    if (_currentCallType == CallTypes.Type_Being_Called &&
        _currentCallStatus == CallStatus.calling) {
      await _tRTCCallingService.reject();
    } else {
      await _tRTCCallingService.hangup();
    }
    widget.close();
  }

  //接听
  onAcceptCall() async {
    await _tRTCCallingService.accept();
    _safeSetState(() {
      _currentCallStatus = CallStatus.answer;
    });
  }

  _onSwitchAudioTap() {
    _tRTCCallingService.closeCamera();
    _safeSetState(() {
      _callingScenes = CallingScenes.Audio;
    });
  }

  getTopBarWidget() {
    final showName = TRTCC2cUtils.getShowName(_remoteUserInfo);
    bool isCalling = _currentCallStatus == CallStatus.calling ? true : false;
    var topWidget = Positioned(
      left: 0,
      top: _callingScenes == CallingScenes.Video ? 64 : 250,
      width: MediaQuery.of(context).size.width,
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: isCalling
            ?
            // 拨打中的情况
            [
                Text(
                  showName,
                  style: const TextStyle(
                    fontSize: 24,
                    color: Colors.white,
                    fontWeight: FontWeight.bold,
                  ),
                ),
                Text(
                  widget.isInitiationCalling ? "等待对方接受邀请..." : "邀请您通话...",
                  style: const TextStyle(fontSize: 12, color: Colors.white),
                )
              ]
            :
            // 已经接听的情况
            _callingScenes == CallingScenes.Video
                ? []
                : [
                    //1V1语音通话显示名字
                    Text(
                      showName,
                      style: const TextStyle(
                        fontSize: 24,
                        color: Colors.white,
                        fontWeight: FontWeight.bold,
                      ),
                    ),
                  ],
      ),
    );
    return topWidget;
  }

  getButtomWidget() {
    // 什么情况下展示什么图标
    bool isAudioScence = _callingScenes == CallingScenes.Audio;

    //语音聊天时的button按钮
    var audioBtnList = [
      isAudioScence && _currentCallStatus == CallStatus.answer
          ? ExtendButton(
              imgUrl: _isMicrophoneOff
                  ? "images/microphone-off.png"
                  : "images/microphone-on.png",
              tips: "麦克风",
              onTap: () {
                onMicrophoneTap();
              },
            )
          : const SizedBox(),
      ExtendButton(
        imgUrl: "images/hangup.png",
        tips: "挂断",
        onTap: () {
          onHangUpCall();
        },
      ),
      isAudioScence && _currentCallStatus == CallStatus.answer
          ? ExtendButton(
              imgUrl: _isHandsFree
                  ? "images/trtccalling_ic_handsfree_enable.png"
                  : "images/trtccalling_ic_handsfree_disable.png",
              tips: "扬声器",
              onTap: () {
                onHandsfreeTap();
              },
            )
          : const SizedBox(
              height: 0,
              width: 0,
            )
    ];
    if (_currentCallType == CallTypes.Type_Being_Called &&
        _currentCallStatus == CallStatus.calling) {
      audioBtnList.insert(
        2,
        const SizedBox(
          height: 0,
          width: 0,
        ),
      );
      audioBtnList.insert(
        3,
        ExtendButton(
          imgUrl: "images/trtccalling_ic_dialing.png",
          tips: "接听",
          onTap: () {
            onAcceptCall();
          },
        ),
      );
    }

    var videoBtnList = !isAudioScence && _currentCallStatus == CallStatus.answer
        ? [
            ExtendButton(
              imgUrl: _isMicrophoneOff
                  ? "images/microphone-off.png"
                  : "images/microphone-on.png",
              tips: "麦克风",
              onTap: () {
                onMicrophoneTap();
              },
            ),
            ExtendButton(
              imgUrl: _isHandsFree
                  ? "images/trtccalling_ic_handsfree_enable.png"
                  : "images/trtccalling_ic_handsfree_disable.png",
              tips: "扬声器",
              onTap: () {
                onHandsfreeTap();
              },
            ),
            ExtendButton(
              imgUrl: _isCameraOff
                  ? "images/camera-off.png"
                  : "images/camera-on.png",
              tips: "摄像头",
              onTap: () {
                onCameraTap();
              },
            )
          ]
        : [const SizedBox()];
    var buttomWidget = Positioned(
      left: 0,
      bottom: 50,
      width: MediaQuery.of(context).size.width,
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          Container(
              margin: const EdgeInsets.only(bottom: 10),
              child: Column(
                children: [
                  _currentCallStatus == CallStatus.answer
                      ? Container(
                          margin: const EdgeInsets.only(bottom: 10),
                          child: Text(
                            _hadCallingTime,
                            style: const TextStyle(color: Colors.white),
                          ),
                        )
                      : const SizedBox(),
                  !isAudioScence
                      ? ExtendButton(
                          imgUrl: "images/switchToAudio.png",
                          imgHieght: 18,
                          imgColor: const Color.fromRGBO(125, 123, 123, 1.0),
                          tips: "切到语音通话",
                          onTap: () {
                            _onSwitchAudioTap();
                          },
                        )
                      : const SizedBox(),
                ],
              )),
          Container(
            margin: const EdgeInsets.only(
              bottom: 20,
            ),
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
              children: videoBtnList,
            ),
          ),
          !isAudioScence && _currentCallStatus == CallStatus.answer
              ? Row(
                  children: [
                    const SizedBox(
                      width: 40,
                    ),
                    ...audioBtnList,
                    const SizedBox(
                      width: 16,
                    ),
                    InkWell(
                      onTap: () {
                        _onSwitchCamera();
                      },
                      child: Image.asset(
                        'images/switch-camera.png',
                        package: 'tim_ui_kit_calling_plugin',
                        height: 25,
                        color: const Color.fromRGBO(125, 123, 123, 1.0),
                      ),
                    ),
                  ],
                  mainAxisAlignment: MainAxisAlignment.center,
                )
              : Row(
                  mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                  children: audioBtnList,
                ),
        ],
      ),
    );
    return buttomWidget;
  }

  _safeSetState(callBack) {
    setState(() {
      if (mounted) {
        callBack();
      }
    });
  }

  // 占满全屏幕的大窗
  Widget getBigVideo() {
    if (_callingScenes == CallingScenes.Audio) return Container();
    bool nowIsLocalView = true; //判断当前大窗口是否显示本地摄像头
    if (_currentCallStatus == CallStatus.calling) {
      nowIsLocalView = true;
    } else {
      //已经接听
      if (isChangeBigSmallVideo) {
        nowIsLocalView = true;
      } else {
        nowIsLocalView = false; //远端画面
      }
    }
    var opacityVal = nowIsLocalView
        ? TRTCC2cUtils.getOpacityByVis(!_isCameraOff)
        : TRTCC2cUtils.getOpacityByVis(_remoteUserAvailable);
    return _callingScenes == CallingScenes.Video
        ? AnimatedOpacity(
            duration: const Duration(milliseconds: 100),
            opacity: opacityVal,
            child: TRTCCloudVideoView(
              key: const ValueKey("_bigVideoViewId"),
              viewType: TRTCCloudDef.TRTC_VideoView_TextureView,
              onViewCreated: (viewId) async {
                _bigVideoViewId = viewId;
                if (_callingScenes == CallingScenes.Video) {
                  await _tRTCCallingService.openCamera(
                      _isFrontCamera, _bigVideoViewId);
                }
              },
            ),
          )
        : Container();
  }

  // 右上角小窗
  getSmallVideoContainer() {
    BoxDecoration selfPortraitDecorat = BoxDecoration(
      borderRadius: BorderRadius.circular(8),
      image: DecorationImage(
        image: NetworkImage(_remoteUserInfo.faceUrl ?? ""),
        fit: BoxFit.cover,
      ),
    );
    if (_callingScenes == CallingScenes.Audio) {
      return Container(
          height: 110,
          width: 110,
          child: Container(),
          decoration: selfPortraitDecorat);
    }
    bool nowIsRemoteView = false; //判断当前小窗口是否显示远端画面
    if (_currentCallStatus == CallStatus.calling) {
      nowIsRemoteView = true;
    } else {
      //已经接听
      if (isChangeBigSmallVideo) {
        nowIsRemoteView = true;
      } else {
        nowIsRemoteView = false; //本地摄像头
      }
    }
    return Container(
      height: _currentCallStatus == CallStatus.calling ? 110 : 216,
      width: 110,
      child: _currentCallStatus == CallStatus.answer
          ? AnimatedOpacity(
              duration: const Duration(milliseconds: 100),
              opacity: nowIsRemoteView
                  ? TRTCC2cUtils.getOpacityByVis(_remoteUserAvailable)
                  : TRTCC2cUtils.getOpacityByVis(!_isCameraOff),
              child: TRTCCloudVideoView(
                key: const ValueKey("_smallVideoViewId"),
                viewType: TRTCCloudDef.TRTC_VideoView_TextureView,
                onViewCreated: (viewId) async {
                  _smallVideoViewId = viewId;
                  await _tRTCCallingService.updateLocalView(_smallVideoViewId);
                },
              ),
            )
          : Container(),
      decoration: _currentCallStatus == CallStatus.calling
          ? selfPortraitDecorat
          : const BoxDecoration(),
    );
  }

  _changeVideoView() {
    if (_callingScenes == CallingScenes.Audio ||
        _currentCallStatus == CallStatus.calling) return;

    isChangeBigSmallVideo = !isChangeBigSmallVideo;
    //为false的时候，在已接听状态的时候。小画面显示本地视频，大画面显示远端视频。
    if (isChangeBigSmallVideo) {
      _tRTCCallingService.updateLocalView(_bigVideoViewId);
      _tRTCCallingService.updateRemoteView(_remoteUserInfo.userID!,
          TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_BIG, _smallVideoViewId);
    } else {
      _tRTCCallingService.updateLocalView(_smallVideoViewId);
      _tRTCCallingService.updateRemoteView(_remoteUserInfo.userID!,
          TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_BIG, _bigVideoViewId);
    }
  }

  Widget remoteInfoPanel() {
    return Positioned(
        top: _callingScenes == CallingScenes.Video
            ? _smallViewTop - 60
            : _smallViewTop,
        right: _callingScenes == CallingScenes.Video
            ? _smallViewRight
            : MediaQuery.of(context).size.width / 2 - 110 / 2,
        child: GestureDetector(
          onDoubleTap: () {
            _changeVideoView();
          },
          onPanUpdate: (DragUpdateDetails e) {
            //用户手指滑动时，更新偏移，重新构建
            if (_callingScenes == CallingScenes.Video) {
              _safeSetState(() {
                _smallViewRight -= e.delta.dx;
                _smallViewTop += e.delta.dy;
              });
            }
          },
          child: SizedBox(
            width: 110,
            child: getSmallVideoContainer(),
          ),
        ));
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: WillPopScope(
        onWillPop: () async {
          return true;
        },
        child: Stack(
          alignment: Alignment.topLeft,
          fit: StackFit.expand,
          children: [
            Container(
              color: const Color(0xFF444444),
              child: getBigVideo(),
            ),
            // 对方头像
            remoteInfoPanel(),
            // 对方用户名信息和提示信息
            getTopBarWidget(),
            // 底部挂断按钮等
            getButtomWidget(),
          ],
        ),
      ),
    );
  }
}
